@@ -4,21 +4,20 @@
 """ Package processing support. """
 
 import enum
-import io
 import itertools
 import re
-import subprocess
 import typing
 
 from pathlib import Path
 
 import lxml.etree
 
-# need to preload it to fix pkgcheck.reporters import error
-__import__('pkgcheck.checks')
-from pkgcheck.reporters import PickleStream
+import pkgcheck
 from pkgcheck.results import Result
-from pkgcheck.checks.visibility import _NonsolvableDeps
+try:
+    from pkgcheck.checks.visibility import NonsolvableDeps
+except ImportError:
+    from pkgcheck.checks.visibility import _NonsolvableDeps as NonsolvableDeps
 
 import pkgcore.ebuild.domain
 import pkgcore.ebuild.ebuild_src
@@ -91,10 +90,14 @@ class PackageInvalid(PackageMatchException):
 
 
 class KeywordNotSpecified(PackageMatchException):
-    pass
+    def __init__(self,
+                 pkgs: typing.List[str],
+                 message: str):
+        self.pkgs = pkgs
+        return super().__init__(message)
 
 
-class KeywordNoneLeft(KeywordNotSpecified):
+class KeywordNoneLeft(Exception):
     pass
 
 
@@ -241,8 +244,8 @@ def match_package_list(repo: UnconfiguredTree,
     keyworded_already = False
     filtered = False
     yielded = False
-    no_potential_keywords = False
-    no_keywords = False
+    no_potential_keywords = []
+    no_keywords = []
 
     prev_keywords = None
     for line in bug.atoms.splitlines():
@@ -309,9 +312,9 @@ def match_package_list(repo: UnconfiguredTree,
 
         if not keywords:
             if not get_suggested_keywords(repo, pkg, streq):
-                no_potential_keywords = True
+                no_potential_keywords.append(sdep)
             else:
-                no_keywords = True
+                no_keywords.append(sdep)
             yield PackageKeywords(pkg, keywords)
             continue
         prev_keywords = keywords
@@ -349,7 +352,9 @@ def match_package_list(repo: UnconfiguredTree,
         yielded = True
 
     if no_keywords:
-        raise KeywordNotSpecified('incomplete keywords')
+        raise KeywordNotSpecified(no_keywords,
+                                  f'incomplete keywords for packages: '
+                                  f'{" ".join(no_keywords)})')
     elif no_potential_keywords:
         # report KeywordNoneLeft only if no other entries were reported
         # otherwise, the bug is still considered interesting
@@ -357,7 +362,9 @@ def match_package_list(repo: UnconfiguredTree,
             raise KeywordNoneLeft('package keywords in line with other '
                                   'versions and none specified')
         else:
-            raise KeywordNotSpecified('incomplete keywords')
+            raise KeywordNotSpecified(no_potential_keywords,
+                                      f'incomplete keywords for packages: '
+                                      f'{" ".join(no_potential_keywords)}')
     if not yielded:
         if filtered:
             raise PackageListEmpty('no packages match requested arch')
@@ -474,16 +481,20 @@ def check_dependencies(repo: UnconfiguredTree,
 
     for keywords, packages in itertools.groupby(tuples, lambda x: x[1]):
         package_strs = list((str(x[0].versioned_atom) for x in packages))
-        args = ['pkgcheck', 'scan', '-c', 'VisibilityCheck',
-                '-R', 'PickleStream', '-a', ','.join(keywords)] + package_strs
-        sp = subprocess.Popen(args,
-                              cwd=repo.location,
-                              stdout=subprocess.PIPE,
-                              stderr=subprocess.PIPE)
-        sout, serr = sp.communicate()
+        results = pkgcheck.scan(['-c', 'VisibilityCheck',
+                                 '-p', 'stable,dev',
+                                 '-a', ','.join(keywords),
+                                 '-r', repo.location,
+                                 ] + package_strs)
 
-        for r in PickleStream.from_file(io.BytesIO(sout)):
+        results = list(results)
+        for r in results:
             if r.name.startswith('NonsolvableDeps'):
+                # workaround a bug (or feature?) in pkgcheck-0.8*
+                # that causes the checks to be done against all versions
+                pkgstr = f'={r.category}/{r.package}-{r.version}'
+                if pkgstr not in package_strs:
+                    continue
                 ret = False
                 errors.append(r)
 
@@ -552,12 +563,35 @@ def is_allarches(pkg: pkgcore.ebuild.ebuild_src.package
     return False
 
 
-def result_group_key(r: _NonsolvableDeps) -> tuple:
+def can_allarches_for_keywords(repo: UnconfiguredTree,
+                               tuples: PackageKeywordsIterable,
+                               ) -> bool:
+    """
+    Verify whether `pkg` is actually suitable for ALLARCHES.
+
+    Return True if all packages in tuples have at least one stable
+    version on each requested architecture, False otherwise.
+    """
+
+    for req_package, keywords in tuples:
+        keywords_left = set(keywords)
+        for p in repo.itermatch(req_package.unversioned_atom):
+            # NB: we don't need to filter ~arch or -arch keywords, they
+            # just won't match anything
+            keywords_left.difference_update(p.keywords)
+        # if we couldn't match at least one keyword, we can't
+        # do ALLARCHES
+        if keywords_left:
+            return False
+    return True
+
+
+def result_group_key(r: NonsolvableDeps) -> tuple:
     """Key used to group pkgcheck results"""
     return (r.category, r.package, r.version)
 
 
-def result_sort_key(r: _NonsolvableDeps) -> tuple:
+def result_sort_key(r: NonsolvableDeps) -> tuple:
     """Key used to sort pkgcheck results"""
     return (r.category, r.package, r.version,
             r.keyword, r.attr, r.profile)
@@ -569,7 +603,7 @@ def format_results(issues: typing.Iterable[Result]
     Format pkgcheck results `issues` and yield list of result lines
     """
     for r in issues:
-        assert isinstance(r, _NonsolvableDeps)
+        assert isinstance(r, NonsolvableDeps)
     for key, values in itertools.groupby(
             issues,
             key=result_group_key):
